/******************************************************************************/
/* collator-1 - network traffic visulaisation                                 */
/* copyright (c) 2004 jamie twycross, jpt@cs.nott.ac.uk                       */
/* released under the gnu general public license v2                           */
/******************************************************************************/

static const char RCSid[] = "$Id: collator-1.c,v 1.5 2004/03/05 01:20:19 jpt Exp $";
const char *RCScruft7 = RCSid;

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <errno.h>
#include <math.h>
#include "collcoll.h"

#define INCLUDE_CICHLID_UTILS
#include <cichlid_server.h>

#define NUM_VERTICES	8
#define NUM_EDGES	7
#define SPHERE_SIZE	10.0
#define CUBE_SIZE	7.0
#define PYRAMID_SIZE	10.0
#define EDGE_SIZE	1.0
#define EDGE_MAX	6.0
#define VERTEX_COLOR_1	{0, 0, 255}
#define VERTEX_COLOR_2	{255, 0, 0}
#define VERTEX_COLOR_3	{255, 255, 0}
#define EDGE_COLOR_1	{0, 255, 0}
#define LABEL_COLOR	255
#define SPACE_SIZE	{200.0, 200.0, 200.0}
#define GAMMA		0.005

Collector collectors[] = {
	{"sister", 22450},
	{"outsider", 22451},
	{NULL}
};

static struct DataSet *dset;
static const float rspace[3] = SPACE_SIZE;

static struct VtxInfo vertices[NUM_VERTICES] =
{
	{ {-30.0, -25.0, -15.0}, VD_PYRAMID_3, PYRAMID_SIZE, VERTEX_COLOR_3 }, // cat
	{ {-30.0, -5.0, -40.0}, VD_SPHERE, SPHERE_SIZE, VERTEX_COLOR_2 },  // ringo
	{ {-12.66, -35.0, -40.0}, VD_SPHERE, SPHERE_SIZE, VERTEX_COLOR_2 },  // george
	{ {-47.33, -35.0, -40.0}, VD_SPHERE, SPHERE_SIZE, VERTEX_COLOR_2 },  // wilde
	{ {0.0, 0.0, 0.0}, VD_CUBE, CUBE_SIZE, VERTEX_COLOR_1 },  // jordan
	{ {0.0, 0.0, 30.0}, VD_CUBE, CUBE_SIZE, VERTEX_COLOR_1 },  // janice
	{ {25.0, 25.0, -15.0}, VD_PYRAMID_3, PYRAMID_SIZE, VERTEX_COLOR_3 },  // moh
	{ {25.0, 25.0, -40.0}, VD_SPHERE, SPHERE_SIZE, VERTEX_COLOR_2 }  // reid
};

static struct EdgeInfo edges[NUM_EDGES] =
{
	{ {5, 4}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 },  // janice - jordan
	{ {4, 6}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 },  // jordan - moh
	{ {6, 7}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 },  // moh - reid
	{ {0, 4}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 },  // cat - jordan
	{ {0, 1}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 },  // cat - ringo
	{ {0, 2}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 },  // cat - george
	{ {0, 3}, ED_NONE, ER_CYLINDER, EDGE_SIZE, EDGE_COLOR_1 }  // cat - wilde
};

static char *vertex_labels[NUM_VERTICES] =
{
	"cat",
	"ringo",
	"george",
	"wilde",
	"jordan",
	"janice",
	"moh",
	"reid"
};

static void data_init(void)
{
	unsigned int i;

	veset_set_vtcs(dset, 0, &vertices[0], NUM_VERTICES);
	veset_set_edges(dset, 0, &edges[0], NUM_EDGES);

	for(i = 0; i < NUM_VERTICES; i++)
		veset_set_vlabel(dset, i, vertex_labels[i], NULL);

	for(i = 0; i < NUM_EDGES; i++)
		set_data_nolock(i, 0.0);
}

static inline float scale_load(float a)
{
	return((2.0 / (1.0 + exp(-1.0 * GAMMA * a))) - 1.0);
}

static void data_calc(void)
{
	register unsigned int i;

	poll_collectors();
	for(i = 0; i < NUM_EDGES; i++)
		edges[i].size = EDGE_SIZE + EDGE_MAX * scale_load(get_data_nolock(i));
		
	veset_set_vtcs(dset, 0, &vertices[0], NUM_VERTICES);
	veset_set_edges(dset, 0, &edges[0], NUM_EDGES);
}

int main(int argc, char **argv)
{
	/* initialise data storage  */
	init_data(NUM_EDGES);

	/* connect to collectors */
	init_collectors(collectors);
	
	/* initialise cichlid */
	dsInit(&argc, argv, TB_FALSE);
	dset = veset_new("collator-1", 0, NUM_VERTICES, NUM_EDGES, rspace, \
			CS_RECTANGULAR);
	data_init();
	dsSetDataSet(dset);
	dsDataFunc(data_calc);
	dsDisconnectFunc(term_collectors);

	/* start cichlid */
	dsMainLoop();
	
	/* close connections to collectors */
	term_collectors();
	
	return EXIT_SUCCESS;
}
