#include <stdio.h>
#include <stdlib.h>
#include "collcoll.h"
#include "netmisc.h"

#define NUM_LINKS	8
#define NUM_EDGES	7
#define DEVICE		"eth0"

static unsigned char link_info[NUM_LINKS][2][6] =
{
	{{0x00,0x0C,0x41,0x27,0x0B,0xE1},   // jordan - ringo
	{0x00,0x50,0x04,0x4A,0x2C,0x41}},
	{{0x00,0x0C,0x41,0x27,0x0B,0xE1},   // jordan - george
	{0x00,0x0A,0x5E,0x20,0x4A,0x82}},
	{{0x00,0x0C,0x41,0x27,0x0B,0xE1},   // jordan - wilde
	{0x00,0x50,0x04,0x4A,0x2C,0x3C}},
	{{0x00,0x50,0x04,0x4A,0x2C,0x41},   // ringo - george
	{0x00,0x0A,0x5E,0x20,0x4A,0x82}},
	{{0x00,0x0A,0x5E,0x20,0x4A,0x82},   // george - wilde
	{0x00,0x50,0x04,0x4A,0x2C,0x3C}},
	{{0x00,0x50,0x04,0x4A,0x2C,0x3C},   // wilde - ringo
	{0x00,0x50,0x04,0x4A,0x2C,0x41}},
	{{0x00,0x50,0x04,0x4A,0x2C,0x81},  // janice - jordan
	{0x00,0x50,0xDA,0x43,0x5C,0x93}},
	{{0x00,0x10,0x4B,0xB5,0xEA,0x3C},  // jordan - reid
	{0x00,0x90,0x27,0x0E,0x93,0x35}}
};

void packet_handler(u_char *dummy, const struct pcap_pkthdr *pkthdr, \
		const u_char *packet)
{
	register unsigned int link_num;
	register float load;
	unsigned char *shaddr, *dhaddr;

	/* sanity check packet */
	if(pkthdr->caplen < 12)
		return;

	/* see if it's interesting */
	dhaddr = (unsigned char *) packet;
	shaddr = (unsigned char *) (packet + 6);
	for(link_num = 0; link_num < NUM_LINKS; link_num++)
	{
		if((compare_mac(link_info[link_num][0], shaddr) && \
				compare_mac(link_info[link_num][1], dhaddr)) ||
				(compare_mac(link_info[link_num][1], shaddr) && \
				compare_mac(link_info[link_num][0], dhaddr)))
			break;
	}
	if(link_num == NUM_LINKS)
		return;

	/* interesting, so add to edge data */
	load = (float) pkthdr->len;
	switch(link_num)
	{
		case 0:
			add_data(3, load);
			add_data(4, load);
			break;

		case 1:
			add_data(3, load);
			add_data(5, load);
			break;

		case 2:
			add_data(3, load);
			add_data(6, load);
			break;

		case 3:
			add_data(4, load);
			add_data(5, load);
			break;

		case 4:
			add_data(5, load);
			add_data(6, load);
			break;

		case 5:
			add_data(6, load);
			add_data(4, load);
			break;

		case 6:
			add_data(0, load);
			break;

		case 7:
			add_data(1, load);
			add_data(2, load);
			break;

		default:
			break;
	}
}

int main(int argc, char **argv)
{
	unsigned short int port;
	char *device = DEVICE;

	/* parse command line */
	switch(argc)
	{
		case 2:
			port = atoi(argv[1]); 
			break;

		case 3:
			device = argv[1];
			port = atoi(argv[2]); 
			break;
			
		default:
			fprintf(stderr, \
				"usage: collector-1 [INTERFACE] PORT\n");
			return EXIT_FAILURE;
	}

	/* initialise the data storage */
	init_data(NUM_EDGES);
	
	/* initialise the data server */
	init_server(port);

	/* start the sniffer */
	start_sniffer(packet_handler, device);

	/* start the data server */
	start_server();
	
	return EXIT_SUCCESS;
}
